Свойство **colsHead** возвращает преобразованный массив столбцов.

Этот свойство имеет режим доступа только для чтения, т.е. оно фактически возвращает преобразованный массив **columns**, в котором сделаны следующие изменения:

1. Исключены столбцы, которые пользователь спрятал во время работы.
1. Добавлен пустой столбец для заполнения пустой области контейнера, если столбцы не выравниваются по всей его ширине.
1. Добавлен столбец для вывода кнопок выбора строк при условии, что свойство **allowCheck** установлено в значение **true** и есть строки, в которых не отображаются деревья.
1. Массив сортируется по свойству **order**, который задан либо по умолчанию, либо изменен пользователем во время работы с компонентом.
1. Для каждого фиксированного столбца задается его расположение внутри контейнера.
1. Каждому столбцу добавляется свойство id, в котором записывается его текущий порядок после всех сделанных изменений.

Например:

```javascript _run_line_edit_loadoda_[my-component.js]_h=160_
import '/components/grids/table/table.js';
ODA({
    is: 'my-component',
    template: `
        <label>Показать кнопки выбора <input type="checkbox" ::value="allowCheck" ></label> <br>
        <div>Количество преобразованных столбцов: {{$refs.tree.colsHead.length}}
        </div>
        <div>Столбец: {{JSON.stringify($refs.tree.colsHead[0])}}</div>
        <oda-table ref="table" :allow-check :data-set show-header :columns></oda-table>
    `,
    props: {
        columns: [
            {name: 'col0', label: 'Фиксированный столбец', fix: true},
            {name: 'col1', label: 'Столбец 1'},
            {name: 'col2', label: 'Столбец 2'}
        ],
        dataSet: [
            {col1: "1 Строка", col2: "1"},
            {col1: "2 Строка", col2: "2"},
            {col1: "3 Строка", col2: "3"}
        ],
        allowCheck: false
    }
});
```
