Свойство **expanding** позволяет определить завершен ли процесс свертывания или развертывания узлов дерева на данный момент.

Данное свойство работает только со столбцами таблицы, которые находятся в режиме **treeMode**, т.е. отображения данных в виде деревьев. Процесс развертывания узлов в них носит асинхронный характер и может занимать значительное время. В начале этого процесса свойство **expanding** устанавливается в значение **true**, сигнализируя о том, что процесс развертывания запущен. После того как узлы дерева будут развернуты, свойство **expanding** автоматически устанавливается в значение **false**, свидетельствуя о том, что процесс развертывания узлов уже завершен.

Аналогично это свойство работает и в момент свертывания узлов, позволяя узнать, что этот процесс начался, но еще не завершился.

Например:

```javascript _run_line_edit_loadoda_[my-component.js]_h=240_
import '/components/grids/table/table.js';
ODA({
    is: 'my-component',
    template: `
        <label>{{text}}<input type="checkbox" ::value="expanded" ></label> <br>
        <label>Свойство expanding: <input type="checkbox" ::value="$refs.table.expanding" ></label> <br>
        <oda-table ref="table" :data-set :columns show-header col-lines auto-width></oda-table>
    `,
    props: {
        columns: [
            {name: 'col1', label: 'Столбец 1'},
            {name: 'col2', label: 'Столбец 2', treeMode: true}
        ],
        dataSet: [
            {col1: "1 строка", col2:"1 узел"},
            {col1: "2 строка", col2:"2 узел",
                items:[
                    {col2: "2.1 узел"},
                    {col2: "2.2 узел",
                    items:[
                        {col2:"2.2.1 узел"},
                        {col2:"2.2.2 узел"},
                        {col2:"2.2.3 узел"},
                    ]},
                    {col2:"2.3 узел"}
                ]
            },
        ],
        text: 'Узлы свернуты: ',
        expanded: {
            default: false,
            set(n) {
                this.text = n ? 'Узлы развернуты: ' : 'Узлы свернуты: ';
                n ? this.$refs.table.expandAll() : this.$refs.table.collapseAll();
            }
        }
    }
});
```

Если узлов в дереве не много, то заметить изменение значения этого свойства во время свертывания или развертывания узлов практически не возможно.

Кроме этого, если это свойство установить в ручном режиме в значение **true**, то функции свертывания и развертывания работать перестанут. Они будут считать, что предыдущий процесс еще не завершен, и никаких действий выполнять не будут.
