Свойство **colsHead** возвращает преобразованный массив столбцов.

Этот свойство имеет режим доступа только для чтения, т.е. оно фактически возвращает массив **columns**, в котором сделаны следующие изменения:

1. Исключены столбцы, которые пользователь спрятал во время работы.
1. Добавлен пустой столбец для заполнения пустой области контейнера, если столбцы не выравниваются по всей его ширине.
1. Добавлен столбец для вывода кнопок выбора строк при условии, что свойство **allowCheck** установлено в значение **true** и есть строки, в которых не отображаются деревья.
1. Массив отсортировывается по свойству **order**, который задан либо по умолчанию, либо изменен пользователем во время работы с компонентом.
1. Для каждого фиксированного столбца задается его расположение внутри контейнера.
1. Каждому столбцу добавляется свойство **id**, в котором записывается его текущий порядок после всех сделанных изменений.

Например:

```javascript _run_line_edit_loadoda_[my-component.js]_h=200_
import '/components/grids/tree/tree.js';
ODA({
    is: 'my-component',
    template: `
        <label>Показать заголовок: <input type="checkbox" ::value="showHeader" ></label>
        <div>Количество преобразованных столбцов: {{$refs.tree.colsHead.length}}
        </div>
        <div>Столбец: {{JSON.stringify($refs.tree.colsHead[0])}}</div>
        <oda-tree ref="tree" :data-set :show-header :columns></oda-tree>
    `,
    props: {
        columns: [
            {name: 'name', label: 'Мой столбец', treeMode: true}
        ],
        dataSet: [
            {name: "1 строка"},
            {name:"2 строка",
                items:[
                    {name:"2.1 строка"},
                    {name:"2.2 строка",
                    items:[
                        {name:"2.2.1 строка"},
                        {name:"2.2.2 строка"},
                        {name:"2.2.3 строка"},
                    ]},
                    {name:"2.3 строка"}
                ]
            },
            {name:"3 строка"},
        ],
        showHeader: false
    }
});
```

Данное свойство используется, когда столбцы задаются с помощью массива **columns**. Для столбца по умолчанию никаких преобразований не происходит, и свойство  **colsHead** возвращает значение **undefined**.

Для отображения дерева нужен только один столбец, который всегда виден. Поэтому свойство **colsHead** можно не использовать.
