Свойство **columns** предназначено для хранения столбца таблицы, в котором отображаются узлы дерева.

Для использования этого свойства необходимо задать массив, содержащий объект, свойства которого определяют следующие характеристики столбца:

1. **name** – название столбца.

    Название столбца определяет имя свойства, значения которого будут отображаться в виде узлов дерева из набора данных **dataSet**.
1. **label** – надпись заголовка столбца.

    Если это свойство не указано, то в заголовке столбца название будет формироваться на основе свойств **name**.
1. **fix** – вид столбца (фиксированный или нет).

    Если это свойство имеет любое значение, которое можно привести к значению **true**, то все ячейки столбца будут отображены другим стилем, предусмотренным для фиксированных столбцов таблицы.
1. **treeMode** – режим отображения данных столбца в виде дерева.

    Значение этого свойства обязательно должно иметь значение **true**, чтобы данные отображались внутри столбца в виде дерева.

Например:

```javascript _run_line_edit_loadoda_[my-component.js]_h=140_
import '/components/grids/tree/tree.js';
ODA({
    is: 'my-component',
    template: `
        <div>Количество столбцов: {{$refs.tree.columns.length}}</div>
        <div>Название столбца: {{$refs.tree.columns[0].name}}</div>
        <oda-tree ref="tree" :data-set :columns show-header></oda-tree>
    `,
    props: {
        columns: [
            {name: 'name', label: 'Мой столбец', treeMode: true}
        ],
        dataSet: [
            {name: "1 строка"},
            {name:"2 строка",
                items:[
                    {name:"2.1 строка", fix: true},
                    {name:"2.2 строка",
                    items:[
                        {name:"2.2.1 строка"},
                        {name:"2.2.2 строка"},
                        {name:"2.2.3 строка"},
                    ]},
                    {name:"2.3 строка"}
                ]
            },
            {name:"3 строка"},
        ]
    }
});
```

В дереве используется только один столбец для отображения всех узлов. Поэтому в массив **columns** достаточно добавить только один объект с необходимыми свойствами.

Если массив **columns** не будет указан явно, то у дерева автоматически будет создан один столбец по умолчанию. Имя этого столбца будет определяться значением свойства **keyName**, задающим то, какие данные необходимо отображать в узлах дерева из набора **dataSet**. Сам же столбец будет иметь режим отображения дерева **treeNode**:

``` javascript
[{name: this.keyName, treeMode: true}]
```

В этом случае значение столбца возвращается свойством **cols**, а массив **columns** остается пустым.
