Свойство **groups** содержит массив столбцов, по которым осуществляется группировка узлов дерева на данный момент.

Ссылка на любой столбец, который был помещен на группировочную панель, автоматически добавляется в массив **groups**, используя который можно получить доступ к всем группировочным столбцам таблицы.

Например:

```javascript _run_line_edit_loadoda_[my-component.js]_h=220_
import '/components/grids/tree/tree.js';
ODA({
    is: 'my-component',
    template: `
        <oda-tree ref="tree" :data-set :columns show-header show-grouping-panel></oda-tree>
        <div>Группировочные строки</div>
        <div ~for="$refs.tree.groups[0].map.values().next().value"> {{item.value}} </div>
    `,
    props: {
        columns: [
            {name: 'name', label: 'Мой столбец', treeMode: true}
        ],
        dataSet: [
            {name: "1 строка"},
            {name:"2 строка",
                items:[
                    {name:"2.1 строка"},
                    {name:"2.2 строка",
                    items:[
                        {name:"2.2.1 строка"},
                        {name:"2.2.2 строка"},
                        {name:"2.2.3 строка"},
                    ]},
                    {name:"2.3 строка"}
                ]
            },
            {name:"1 строка"},
        ],
        showGroupingPanel: false
    }
});
```

В дереве всегда используется только один столбец таблицы для отображения всех узлов. Поэтому при их группировке в массиве **groups** будет находится только один столбец, у которого не может быть родителей. Корневые узлы этого столбца будут объединяться при условии, что значения их надписей совпадают друг с другом.

Для отображения группировочной панели первоначально необходимо установить свойство **showGroupingPanel** в значение **true**. Кроме этого, заголовки столбцов должны быть доступны для перетаскивания, т.е. свойства **showHeader** должно иметь значение **true**.

По умолчанию массив **groups** будет пустым, так как изначально группировка узлов дерева не проводится. При перемещение столбца на группировочную панель, к нему будет добавлена именованная коллекция **Map**. В ней по уникальному ключу в виде отдельного массива будут храниться все группировочные строки. К таким строкам будет добавлен специальный флажок **$group** со значением **true**, а к обычным узлам дерева будет добавлено свойство **$groupParent** со ссылкой на группировочную строку, в которой они оказались в результате группировки. Кроме этого, группировочная строка будет содержать список всех строк, которые будет находится внутри нее.

Если группировочный столбец удалить с панели, то он будет автоматически удален и из массива **groups**.
