Свойство **iconSize** предназначено для указания размера иконки, отображаемой с левой стороны от надписи у узлов дерева.

Это свойство фактически используется для определения высоты строк, в которых отображаются узлы дерева.

Например:

```javascript _run_line_edit_loadoda_[my-component.js]_h=140_
import '/components/grids/tree/tree.js';
ODA({
    is: 'my-component',
    template: `
       <label> Размер иконки: <input type="number" min="0" max="100" ::value="iconSize">px</label>
        <oda-tree :data-set :icon-size></oda-tree>
    `,
    props: {
        dataSet: [
            {name: "1 строка", template: 'my-template'},
            {name:"2 строка", icon: 'icons:face', template: 'my-template',
                items:[
                    {name:"2.1 строка", icon: 'icons:favorite'},
                    {name:"2.2 строка",
                    items:[
                        {name:"2.2.1 строка", icon: 'icons:android'},
                        {name:"2.2.2 строка"},
                        {name:"2.2.3 строка"},
                    ]},
                    {name:"2.3 строка"}
                ]
            },
            {name:"3 строка",  icon: 'icons:android'},
        ],
        iconSize: 32
    }
});

ODA({
    is: 'my-template',
    template:`
        <style>
            :host{
                @apply --horizontal;
                align-items: center;
            }
        </style>
        <oda-icon :icon="item.icon ? item.icon : 'icons:star'"></oda-icon>
        <span>{{item[column.name]}}</span>`,
    props: {
        column:{},
        item:{}
    }
});
```

Как видно из примера, изменение свойства **iconSize** автоматически приводит к изменению высоты строки одновременно у всех узлов дерева.

Для отображения у узла дерева иконки необходимо задать пользовательский компонент, в котором нужно предусмотреть возможность размещения иконки. Для узлов, которые должны отображаться с помощью этого компонента, необходимо указать свойство **template** со значением имени этого компонента. В данном примере это компонент **my-template**. Только в этом случае у узла дерева будет отображаться иконка, ссылка на которую предусмотрена в свойстве **icon**. Если у узла этого свойства не будет, то у него будет отображаться иконка по умолчанию. В данном примере это иконка с именем **icons:star**.

Если для отображения узлов дерева использовать компонент по умолчанию **oda-table-cell**, то иконки в нем отображаться не будут, даже если они были заданы, так как такая возможность в нем не предусмотрена.
