Свойство **rows** содержит список всех узлов, которые необходимы для обработки узлов в видимой области контейнера дерева.

Данное свойство включает в себя:

1. Все развернутые узлы дерева, расположенные в видимой области контейнера.
1. Родительские узлы, которые сами не попали в видимую область контейнера, но их наследники находятся в ней.
1. Группировочные узлы, которые сами не попали в видимую область контейнера, но их вложенные узлы находятся в ней.

Список узлов дерева в массиве **rows** отличается от списка развернутых узлов дерева **items** только тогда, когда включен режим ленивой обработки данных с помощью свойства **lazy**. В противном случае массивы **items** и **rows** будут абсолютно одинаковыми.

Например:

```javascript _run_line_edit_loadoda_[my-component.js]_h=220_
import '/components/grids/tree/tree.js';
ODA({
    is: 'my-component',
    template: `
        <label>Ленивая обработка <input type="checkbox" ::value="lazy" ></label>
        <div>Количество строк в видимой области: {{$refs.tree.screen.length}}</div>
        <div>Общее количество развернутых строк: {{$refs.tree.items.length}}</div>
        <div>Общее количество обрабатываемых строк: {{$refs.tree.rows.length}}</div>
        <div>Первый узел в списке: {{$refs.tree.rows.length ? $refs.tree.rows[0].name: 'Нет' }}</div>
        <div>Последний узел в списке: {{$refs.tree.rows.length ? $refs.tree.rows[$refs.tree.rows.length - 1].name: 'Нет'}}</div>
        <oda-tree ref="tree" :data-set :lazy style="max-height: 120px"></oda-tree>
    `,
    props: {
        dataSet: [
            {name: "1 строка"},
            {name:"2 строка",
                items:[
                    {name:"2.1 строка"},
                    {name:"2.2 строка",
                    items:[
                        {name:"2.2.1 строка"},
                        {name:"2.2.2 строка"},
                        {name:"2.2.3 строка"},
                    ]},
                    {name:"2.3 строка"}
                ]
            },
            {name:"3 строка"},
        ],
        lazy: false
    }
});
```

По умолчанию свойство **lazy** установлено в значение **false**, т.е. изначально обрабатываются все строки контейнера из массива **items** и свойство **rows** от него ничем не будет отличаться.
