Свойство **sorts** содержит массив столбцов, которые находятся в режиме сортировки.

По умолчанию все узлы в дереве располагаются в том порядке, в котором они были заданы в наборе данных **dataSet**. Однако, разрешив механизм сортировки с помощью свойства **allowSort**, их можно отсортировать по возрастанию или убыванию. Если это свойство установить в значение **true**, то в строке заголовка столбца появится дополнительная кнопка, которая позволит выполнить сортировку узлов в нужном порядке.

Например:

```javascript _run_line_edit_loadoda_[my-component.js]_h=220_
import '/components/grids/tree/tree.js';
ODA({
    is: 'my-component',
    template: `
        <label>Разрешить сортировку <input type="checkbox" ::value="allowSort" ></label>
        <div>Порядок сортировки {{$refs.tree.sorts[0] ? $refs.tree.sorts[0].$sort : 'Нет'}}</div>
        <oda-tree ref="tree" :allow-sort :columns :data-set show-header></oda-tree>
    `,
    props: {
        columns: [
            {name: 'name', label: 'Мой столбец', treeMode: true, $sort='1'}
        ],
        dataSet: [
            {name: "1 строка"},
            {name:"2 строка",
                items:[
                    {name:"2.1 строка"},
                    {name:"2.2 строка",
                    items:[
                        {name:"2.2.1 строка"},
                        {name:"2.2.2 строка"},
                        {name:"2.2.3 строка"},
                    ]},
                    {name:"2.3 строка"}
                ]
            },
            {name:"3 строка"},
        ],
        allowSort: false
    }
});
```

При нажатии на эту кнопку с изображением стрелки вверх или вниз будет последовательно изменяться порядок сортировки узлов дерева. При этом ссылка на столбец, у которого задан порядок сортировки, помещается в массив **sorts**. Кроме этого, в данный столбец добавляется свойство с именем **$sort**, определяющее порядок сортировки узлов. Это свойство может принимать одно из трех возможных значений:

1. $sort = 1 — массив отсортирован по возрастанию.
1. $sort = -1 — массив отсортирован по убыванию.
1. $sort = 0 — массив не отсортирован.

При каждом третьем нажатии кнопки сортировка будет отменена и узлы дерева будут отображаться в том порядке, в котором они были заданы в исходном наборе данных **dataSet**. При этом ссылка на столбец из массива **sorts** автоматически будет удалена, но в самом столбце, хранящемся в массиве **columns**, все равно останется свойство **$sort** со значением 0, свидетельствующем о том, что сортировка по этому столбцу на данный момент не осуществляется.

Для сортировки узлов обязательно необходимо отобразить строку заголовка столбца, задав значение **true** у свойства **showHeader**. Иначе кнопка для сортировки узлов доступна не будет.

По умолчанию столбцы не сортируются, поэтому массив **sorts** будет изначально пустым.
